@@ -26,54 +26,81 @@
 #include <tdebug.h>
 #include <tstring.h>
 #include <tpropertymap.h>
+#include <tagutils.h>
+
 #include "mp4atom.h"
 #include "mp4tag.h"
 #include "mp4file.h"
 
 using namespace TagLib;
 
+namespace
+{
+  bool checkValid(const MP4::AtomList &list)
+  {
+    for(MP4::AtomList::ConstIterator it = list.begin(); it != list.end(); ++it) {
+
+      if((*it)->length == 0)
+        return false;
+
+      if(!checkValid((*it)->children))
+        return false;
+    }
+
+    return true;
+  }
+}  // namespace
+
 class MP4::File::FilePrivate
 {
 public:
-  FilePrivate() : tag(0), atoms(0), properties(0)
-  {
-  }
+  FilePrivate() :
+    tag(0),
+    atoms(0),
+    properties(0) {}
 
   ~FilePrivate()
   {
-    if(atoms) {
-        delete atoms;
-        atoms = 0;
-    }
-    if(tag) {
-        delete tag;
-        tag = 0;
-    }
-    if(properties) {
-        delete properties;
-        properties = 0;
-    }
+    delete atoms;
+    delete tag;
+    delete properties;
   }
 
-  MP4::Tag *tag;
-  MP4::Atoms *atoms;
+  MP4::Tag        *tag;
+  MP4::Atoms      *atoms;
   MP4::Properties *properties;
 };
 
-MP4::File::File(FileName file, bool readProperties, AudioProperties::ReadStyle audioPropertiesStyle)
-    : TagLib::File(file)
+////////////////////////////////////////////////////////////////////////////////
+// static members
+////////////////////////////////////////////////////////////////////////////////
+
+bool MP4::File::isSupported(IOStream *stream)
 {
-  d = new FilePrivate;
-  if(isOpen())
-    read(readProperties, audioPropertiesStyle);
+  // An MP4 file has to have an "ftyp" box first.
+
+  const ByteVector id = Utils::readHeader(stream, 8, false);
+  return id.containsAt("ftyp", 4);
 }
 
-MP4::File::File(IOStream *stream, bool readProperties, AudioProperties::ReadStyle audioPropertiesStyle)
-    : TagLib::File(stream)
+////////////////////////////////////////////////////////////////////////////////
+// public members
+////////////////////////////////////////////////////////////////////////////////
+
+MP4::File::File(FileName file, bool readProperties, AudioProperties::ReadStyle) :
+  TagLib::File(file),
+  d(new FilePrivate())
 {
-  d = new FilePrivate;
   if(isOpen())
-    read(readProperties, audioPropertiesStyle);
+    read(readProperties);
+}
+
+MP4::File::File(IOStream *stream, bool readProperties, AudioProperties::ReadStyle) :
+  TagLib::File(stream),
+  d(new FilePrivate())
+{
+  if(isOpen())
+    read(readProperties);
 }
 
 MP4::File::~File()
@@ -108,40 +135,27 @@ MP4::File::audioProperties() const
   return d->properties;
 }
 
-bool
-MP4::File::checkValid(const MP4::AtomList &list)
-{
-  for(uint i = 0; i < list.size(); i++) {
-    if(list[i]->length == 0)
-      return false;
-    if(!checkValid(list[i]->children))
-      return false;
-  }
-  return true;
-}
-
 void
-MP4::File::read(bool readProperties, Properties::ReadStyle audioPropertiesStyle)
+MP4::File::read(bool readProperties)
 {
   if(!isValid())
     return;
 
   d->atoms = new Atoms(this);
-  if (!checkValid(d->atoms->atoms)) {
+  if(!checkValid(d->atoms->atoms)) {
     setValid(false);
     return;
   }
 
   // must have a moov atom, otherwise consider it invalid
-  MP4::Atom *moov = d->atoms->find("moov");
-  if(!moov) {
+  if(!d->atoms->find("moov")) {
     setValid(false);
     return;
   }
 
   d->tag = new Tag(this, d->atoms);
   if(readProperties) {
-    d->properties = new Properties(this, d->atoms, audioPropertiesStyle);
+    d->properties = new Properties(this, d->atoms);
   }
 }
 
@@ -161,3 +175,28 @@ MP4::File::save()
   return d->tag->save();
 }
 
+bool
+MP4::File::strip(int tags)
+{
+  if(readOnly()) {
+    debug("MP4::File::strip() - Cannot strip tags from a read only file.");
+    return false;
+  }
+
+  if(!isValid()) {
+    debug("MP4::File::strip() -- Cannot strip tags from an invalid file.");
+    return false;
+  }
+
+  if(tags & MP4) {
+    return d->tag->strip();
+  }
+
+  return true;
+}
+
+bool
+MP4::File::hasMP4Tag() const
+{
+  return (d->atoms->find("moov", "udta", "meta", "ilst") != 0);
+}
