@@ -33,7 +33,9 @@ using namespace TagLib;
 class MP4::Item::ItemPrivate : public RefCounter
 {
 public:
-  ItemPrivate() : RefCounter(), valid(true), atomDataType(TypeUndefined) {}
+  ItemPrivate() :
+    valid(true),
+    atomDataType(TypeUndefined) {}
 
   bool valid;
   AtomDataType atomDataType;
@@ -41,8 +43,8 @@ public:
     bool m_bool;
     int m_int;
     IntPair m_intPair;
-    uchar m_byte;
-    uint m_uint;
+    unsigned char m_byte;
+    unsigned int m_uint;
     long long m_longlong;
   };
   StringList m_stringList;
@@ -50,13 +52,14 @@ public:
   MP4::CoverArtList m_coverArtList;
 };
 
-MP4::Item::Item()
+MP4::Item::Item() :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->valid = false;
 }
 
-MP4::Item::Item(const Item &item) : d(item.d)
+MP4::Item::Item(const Item &item) :
+  d(item.d)
 {
   d->ref();
 }
@@ -64,73 +67,76 @@ MP4::Item::Item(const Item &item) : d(item.d)
 MP4::Item &
 MP4::Item::operator=(const Item &item)
 {
-  if(d->deref()) {
-    delete d;
-  }
-  d = item.d;
-  d->ref();
+  Item(item).swap(*this);
   return *this;
 }
 
+void
+MP4::Item::swap(Item &item)
+{
+  using std::swap;
+
+  swap(d, item.d);
+}
+
 MP4::Item::~Item()
 {
-  if(d->deref()) {
+  if(d->deref())
     delete d;
-  }
 }
 
-MP4::Item::Item(bool value)
+MP4::Item::Item(bool value) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->m_bool = value;
 }
 
-MP4::Item::Item(int value)
+MP4::Item::Item(int value) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->m_int = value;
 }
 
-MP4::Item::Item(uchar value)
+MP4::Item::Item(unsigned char value) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->m_byte = value;
 }
 
-MP4::Item::Item(uint value)
+MP4::Item::Item(unsigned int value) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->m_uint = value;
 }
 
-MP4::Item::Item(long long value)
+MP4::Item::Item(long long value) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->m_longlong = value;
 }
 
-MP4::Item::Item(int value1, int value2)
+MP4::Item::Item(int value1, int value2) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->m_intPair.first = value1;
   d->m_intPair.second = value2;
 }
 
-MP4::Item::Item(const ByteVectorList &value)
+MP4::Item::Item(const ByteVectorList &value) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->m_byteVectorList = value;
 }
 
-MP4::Item::Item(const StringList &value)
+MP4::Item::Item(const StringList &value) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->m_stringList = value;
 }
 
-MP4::Item::Item(const MP4::CoverArtList &value)
+MP4::Item::Item(const MP4::CoverArtList &value) :
+  d(new ItemPrivate())
 {
-  d = new ItemPrivate;
   d->m_coverArtList = value;
 }
 
@@ -156,13 +162,13 @@ MP4::Item::toInt() const
   return d->m_int;
 }
 
-uchar
+unsigned char
 MP4::Item::toByte() const
 {
   return d->m_byte;
 }
 
-TagLib::uint
+unsigned int
 MP4::Item::toUInt() const
 {
   return d->m_uint;
@@ -203,4 +209,3 @@ MP4::Item::isValid() const
 {
   return d->valid;
 }
-
