@@ -194,7 +194,7 @@ void TextIdentificationFrame::parseFields(const ByteVector &data)
 
   // read the string data type (the first byte of the field data)
 
-  d->textEncoding = String::Type(data[0]);
+  d->textEncoding = static_cast<String::Type>(data[0]);
 
   // split the byte array into chunks based on the string type (two byte delimiter
   // for unicode encodings)
@@ -220,7 +220,7 @@ void TextIdentificationFrame::parseFields(const ByteVector &data)
 
   unsigned short firstBom = 0;
   for(ByteVectorList::ConstIterator it = l.begin(); it != l.end(); it++) {
-    if(!(*it).isEmpty()) {
+    if(!it->isEmpty() || (it == l.begin() && frameID() == "TXXX")) {
       if(d->textEncoding == String::Latin1) {
         d->fieldList.append(Tag::latin1StringHandler()->parse(*it));
       }
@@ -254,7 +254,7 @@ ByteVector TextIdentificationFrame::renderFields() const
 
   ByteVector v;
 
-  v.append(char(encoding));
+  v.append(static_cast<char>(encoding));
 
   for(StringList::ConstIterator it = d->fieldList.begin(); it != d->fieldList.end(); it++) {
 
