@@ -103,10 +103,10 @@ Ogg::Page::ContainsPacketFlags Ogg::Page::containsPacket(int index) const
   ContainsPacketFlags flags = DoesNotContainPacket;
 
   if(index == d->firstPacketIndex)
-    flags = ContainsPacketFlags(flags | BeginsWithPacket);
+    flags = static_cast<ContainsPacketFlags>(flags | BeginsWithPacket);
 
   if(index == lastPacketIndex)
-    flags = ContainsPacketFlags(flags | EndsWithPacket);
+    flags = static_cast<ContainsPacketFlags>(flags | EndsWithPacket);
 
   // If there's only one page and it's complete:
 
@@ -114,7 +114,7 @@ Ogg::Page::ContainsPacketFlags Ogg::Page::containsPacket(int index) const
      !d->header.firstPacketContinued() &&
      d->header.lastPacketCompleted())
   {
-    flags = ContainsPacketFlags(flags | CompletePacket);
+    flags = static_cast<ContainsPacketFlags>(flags | CompletePacket);
   }
 
   // Or if there is more than one page and the page is
@@ -126,7 +126,7 @@ Ogg::Page::ContainsPacketFlags Ogg::Page::containsPacket(int index) const
            (flags & EndsWithPacket && d->header.lastPacketCompleted()) ||
            (!(flags & BeginsWithPacket) && !(flags & EndsWithPacket))))
   {
-    flags = ContainsPacketFlags(flags | CompletePacket);
+    flags = static_cast<ContainsPacketFlags>(flags | CompletePacket);
   }
 
   return flags;
@@ -208,15 +208,15 @@ List<Ogg::Page *> Ogg::Page::paginate(const ByteVectorList &packets,
 
   static const unsigned int SplitSize = 32 * 255;
 
-  // Force repagination if the packets are too large for a page.
+  // Force repagination if the segment table will exceed the size limit.
 
   if(strategy != Repaginate) {
 
-    size_t totalSize = packets.size();
+    size_t tableSize = 0;
     for(ByteVectorList::ConstIterator it = packets.begin(); it != packets.end(); ++it)
-      totalSize += it->size();
+      tableSize += it->size() / 255 + 1;
 
-    if(totalSize > 255 * 255)
+    if(tableSize > 255)
       strategy = Repaginate;
   }
 
@@ -305,4 +305,10 @@ Ogg::Page::Page(const ByteVectorList &packets,
   }
   d->packets = packets;
   d->header.setPacketSizes(packetSizes);
+
+  // https://xiph.org/ogg/doc/framing.html, absolute granule position:
+  // A special value of '-1' (in two's complement) indicates that no packets
+  // finish on this page.
+  if(!lastPacketCompleted && packets.size() <= 1)
+    d->header.setAbsoluteGranularPosition(-1);
 }
