@@ -27,6 +27,8 @@
 #include <config.h>
 #endif
 
+#if !defined(NDEBUG) || defined(TRACE_IN_RELEASE)
+
 #include "tdebug.h"
 #include "tstring.h"
 #include "tdebuglistener.h"
@@ -43,27 +45,20 @@ namespace TagLib
 
   void debug(const String &s)
   {
-#if !defined(NDEBUG) || defined(TRACE_IN_RELEASE)
-
     debugListener->printMessage("TagLib: " + s + "\n");
-
-#endif
   }
 
   void debugData(const ByteVector &v)
   {
-#if !defined(NDEBUG) || defined(TRACE_IN_RELEASE)
-
-    for(size_t i = 0; i < v.size(); ++i)
-    {
-      std::string bits = std::bitset<8>(v[i]).to_string();
-      String msg = Utils::formatString(
-        "*** [%d] - char '%c' - int %d, 0x%02x, 0b%s\n",
+    for(unsigned int i = 0; i < v.size(); ++i) {
+      const std::string bits = std::bitset<8>(v[i]).to_string();
+      const String msg = Utils::formatString(
+        "*** [%u] - char '%c' - int %d, 0x%02x, 0b%s\n",
         i, v[i], v[i], v[i], bits.c_str());
 
       debugListener->printMessage(msg);
     }
+  }
+}  // namespace TagLib
 
 #endif
-  }
-}
