@@ -41,7 +41,12 @@
  * \note consider conversion via usual char-by-char for loop to avoid UTF16->UTF8->UTF16
  * conversion happening in the background
  */
-#define QStringToTString(s) TagLib::String(s.utf8().data(), TagLib::String::UTF8)
+
+#if defined(QT_VERSION) && (QT_VERSION >= 0x040000)
+#define QStringToTString(s) TagLib::String(s.toUtf8().data(), TagLib::String::UTF8)
+#else
+#define QStringToTString(s) TagLib::String((s).utf8().data(), TagLib::String::UTF8)
+#endif
 
 /*!
  * \relates TagLib::String
@@ -52,7 +57,8 @@
  * conversion happening in the background
  *
  */
-#define TStringToQString(s) QString::fromUtf8(s.toCString(true))
+
+#define TStringToQString(s) QString::fromUtf8((s).toCString(true))
 
 namespace TagLib {
 
@@ -80,8 +86,8 @@ namespace TagLib {
   public:
 
 #ifndef DO_NOT_DOCUMENT
-    typedef std::basic_string<wchar>::iterator Iterator;
-    typedef std::basic_string<wchar>::const_iterator ConstIterator;
+    typedef TagLib::wstring::iterator Iterator;
+    typedef TagLib::wstring::const_iterator ConstIterator;
 #endif
 
     /**
@@ -134,13 +140,21 @@ namespace TagLib {
 
     /*!
      * Makes a deep copy of the data in \a s.
+     *
+     * /note If \a t is UTF16LE, the byte order of \a s will be swapped regardless
+     * of the CPU byte order.  If UTF16BE, it will not be swapped.  This behavior
+     * will be changed in TagLib2.0.
      */
-    String(const wstring &s, Type t = WCharByteOrder);
+    String(const wstring &s, Type t = UTF16BE);
 
     /*!
      * Makes a deep copy of the data in \a s.
+     *
+     * /note If \a t is UTF16LE, the byte order of \a s will be swapped regardless
+     * of the CPU byte order.  If UTF16BE, it will not be swapped.  This behavior
+     * will be changed in TagLib2.0.
      */
-    String(const wchar_t *s, Type t = WCharByteOrder);
+    String(const wchar_t *s, Type t = UTF16BE);
 
     /*!
      * Makes a deep copy of the data in \a c.
@@ -155,7 +169,6 @@ namespace TagLib {
      */
     String(wchar_t c, Type t = Latin1);
 
-
     /*!
      * Makes a deep copy of the data in \a s.
      *
@@ -165,10 +178,7 @@ namespace TagLib {
     String(const char *s, Type t = Latin1);
 
     /*!
-     * Makes a deep copy of the data in \a s.
-     *
-     * \note This should only be used with the 8-bit codecs Latin1 and UTF8, when
-     * used with other codecs it will simply print a warning and exit.
+     * Makes a deep copy of the data in \a v.
      */
     String(const ByteVector &v, Type t = Latin1);
 
@@ -178,7 +188,7 @@ namespace TagLib {
     virtual ~String();
 
     /*!
-     * Returns a deep copy of this String as an std::string.  The returned string 
+     * Returns a deep copy of this String as an std::string.  The returned string
      * is encoded in UTF8 if \a unicode is true, otherwise Latin1.
      *
      * \see toCString()
@@ -186,51 +196,52 @@ namespace TagLib {
     std::string to8Bit(bool unicode = false) const;
 
     /*!
-     * Returns a deep copy of this String as a wstring.  The returned string is 
-     * encoded in UTF-16 (without BOM/CPU byte order).
+     * Returns a deep copy of this String as a wstring.  The returned string is
+     * encoded in UTF-16 (without BOM/CPU byte order), not UTF-32 even if wchar_t
+     * is 32-bit wide.
      *
      * \see toCWString()
      */
     wstring toWString() const;
 
     /*!
-     * Creates and returns a standard C-style (null-terminated) version of this 
-     * String.  The returned string is encoded in UTF8 if \a unicode is true, 
+     * Creates and returns a standard C-style (null-terminated) version of this
+     * String.  The returned string is encoded in UTF8 if \a unicode is true,
      * otherwise Latin1.
-     * 
-     * The returned string is still owned by this String and should not be deleted 
+     *
+     * The returned string is still owned by this String and should not be deleted
      * by the user.
      *
-     * The returned pointer remains valid until this String instance is destroyed 
+     * The returned pointer remains valid until this String instance is destroyed
      * or toCString() is called again.
      *
      * \warning This however has the side effect that the returned string will remain
-     * in memory <b>in addition to</b> other memory that is consumed by this 
+     * in memory <b>in addition to</b> other memory that is consumed by this
      * String instance.  So, this method should not be used on large strings or
      * where memory is critical.  Consider using to8Bit() instead to avoid it.
      *
      * \see to8Bit()
      */
     const char *toCString(bool unicode = false) const;
-    
+
     /*!
-     * Returns a standard C-style (null-terminated) wide character version of 
-     * this String.  The returned string is encoded in UTF-16 (without BOM/CPU byte 
-     * order).
-     * 
-     * The returned string is still owned by this String and should not be deleted 
+     * Returns a standard C-style (null-terminated) wide character version of
+     * this String.  The returned string is encoded in UTF-16 (without BOM/CPU byte
+     * order), not UTF-32 even if wchar_t is 32-bit wide.
+     *
+     * The returned string is still owned by this String and should not be deleted
      * by the user.
      *
-     * The returned pointer remains valid until this String instance is destroyed 
+     * The returned pointer remains valid until this String instance is destroyed
      * or any other method of this String is called.
      *
-     * /note This returns a pointer to the String's internal data without any 
+     * \note This returns a pointer to the String's internal data without any
      * conversions.
      *
      * \see toWString()
      */
     const wchar_t *toCWString() const;
-    
+
     /*!
      * Returns an iterator pointing to the beginning of the string.
      */
@@ -280,7 +291,7 @@ namespace TagLib {
      * Extract a substring from this string starting at \a position and
      * continuing for \a n characters.
      */
-    String substr(uint position, uint n = 0xffffffff) const;
+    String substr(unsigned int position, unsigned int n = 0xffffffff) const;
 
     /*!
      * Append \a s to the current string and return a reference to the current
@@ -288,6 +299,11 @@ namespace TagLib {
      */
     String &append(const String &s);
 
+    /*!
+     * Clears the string.
+     */
+    String &clear();
+
     /*!
      * Returns an upper case version of the string.
      *
@@ -298,12 +314,12 @@ namespace TagLib {
     /*!
      * Returns the size of the string.
      */
-    uint size() const;
+    unsigned int size() const;
 
     /*!
      * Returns the length of the string.  Equivalent to size().
      */
-    uint length() const;
+    unsigned int length() const;
 
     /*!
      * Returns true if the string is empty.
@@ -316,15 +332,25 @@ namespace TagLib {
      * Returns true if this string is null -- i.e. it is a copy of the
      * String::null string.
      *
-     * \note A string can be empty and not null.
+     * \note A string can be empty and not null.  So do not use this method to
+     * check if the string is empty.
+     *
      * \see isEmpty()
+     *
+     * \deprecated Use isEmpty(), do not differentiate between null and empty.
      */
-    bool isNull() const;
+     // BIC: remove
+    TAGLIB_DEPRECATED bool isNull() const;
 
     /*!
      * Returns a ByteVector containing the string's data.  If \a t is Latin1 or
      * UTF8, this will return a vector of 8 bit characters, otherwise it will use
      * 16 bit characters.
+     *
+     * \note If \a t is UTF16, the returned data is encoded in little-endian
+     * format and has a BOM.
+     *
+     * \note The returned data is not null terminated.
      */
     ByteVector data(Type t) const;
 
@@ -340,7 +366,7 @@ namespace TagLib {
     /*!
      * Convert the string to an integer.
      *
-     * If the conversion was successfull, it sets the value of \a *ok to
+     * If the conversion was successful, it sets the value of \a *ok to
      * true and returns the integer. Otherwise it sets \a *ok to false
      * and the result is undefined.
      */
@@ -369,12 +395,12 @@ namespace TagLib {
     /*!
      * Returns a reference to the character at position \a i.
      */
-    wchar &operator[](int i);
+    wchar_t &operator[](int i);
 
     /*!
      * Returns a const reference to the character at position \a i.
      */
-    const wchar &operator[](int i) const;
+    const wchar_t &operator[](int i) const;
 
     /*!
      * Compares each character of the String with each character of \a s and
@@ -388,6 +414,30 @@ namespace TagLib {
      */
     bool operator!=(const String &s) const;
 
+    /*!
+     * Compares each character of the String with each character of \a s and
+     * returns true if the strings match.
+     */
+    bool operator==(const char *s) const;
+
+    /*!
+     * Compares each character of the String with each character of \a s and
+     * returns false if the strings match.
+     */
+    bool operator!=(const char *s) const;
+
+    /*!
+     * Compares each character of the String with each character of \a s and
+     * returns true if the strings match.
+     */
+    bool operator==(const wchar_t *s) const;
+
+    /*!
+     * Compares each character of the String with each character of \a s and
+     * returns false if the strings match.
+     */
+    bool operator!=(const wchar_t *s) const;
+
     /*!
      * Appends \a s to the end of the String.
      */
@@ -454,17 +504,28 @@ namespace TagLib {
      */
     String &operator=(const ByteVector &v);
 
+    /*!
+     * Exchanges the content of the String by the content of \a s.
+     */
+    void swap(String &s);
+
     /*!
      * To be able to use this class in a Map, this operator needed to be
-     * implemented.  Returns true if \a s is less than this string in a bytewise
+     * implemented.  Returns true if \a s is less than this string in a byte-wise
      * comparison.
      */
     bool operator<(const String &s) const;
 
     /*!
      * A null string provided for convenience.
+     *
+     * \warning Do not modify this variable.  It will mess up the internal state
+     * of TagLib.
+     *
+     * \deprecated Use String().
      */
-    static String null;
+     // BIC: remove
+    TAGLIB_DEPRECATED static String null;
 
   protected:
     /*!
@@ -476,61 +537,37 @@ namespace TagLib {
 
   private:
     /*!
-     * Converts a \e Latin-1 string into \e UTF-16(without BOM/CPU byte order) 
-     * and copies it to the internal buffer.
+     * \deprecated This variable is no longer used, but NEVER remove this. It
+     * may lead to a linkage error.
      */
-    void copyFromLatin1(const char *s, size_t length);
-
-    /*!
-     * Converts a \e UTF-8 string into \e UTF-16(without BOM/CPU byte order) 
-     * and copies it to the internal buffer.
-     */
-    void copyFromUTF8(const char *s, size_t length);
-
-    /*!
-     * Converts a \e UTF-16 (with BOM), UTF-16LE or UTF16-BE string into 
-     * \e UTF-16(without BOM/CPU byte order) and copies it to the internal buffer.
-     */
-    void copyFromUTF16(const wchar_t *s, size_t length, Type t);
-
-    /*!
-     * Converts a \e UTF-16 (with BOM), UTF-16LE or UTF16-BE string into 
-     * \e UTF-16(without BOM/CPU byte order) and copies it to the internal buffer.
-     */
-    void copyFromUTF16(const char *s, size_t length, Type t);
-    
-    /*!
-     * Indicates which byte order of UTF-16 is used to store strings internally. 
-     *
-     * \note \e String::UTF16BE or \e String::UTF16LE
-     */
-    static const Type WCharByteOrder;
+     // BIC: remove
+    TAGLIB_DEPRECATED static const Type WCharByteOrder;
 
     class StringPrivate;
     StringPrivate *d;
   };
-}
+}  // namespace TagLib
 
 /*!
  * \relates TagLib::String
  *
  * Concatenates \a s1 and \a s2 and returns the result as a string.
  */
-TAGLIB_EXPORT const TagLib::String operator+(const TagLib::String &s1, const TagLib::String &s2);
+TAGLIB_EXPORT TagLib::String operator+(const TagLib::String &s1, const TagLib::String &s2);
 
 /*!
  * \relates TagLib::String
  *
  * Concatenates \a s1 and \a s2 and returns the result as a string.
  */
-TAGLIB_EXPORT const TagLib::String operator+(const char *s1, const TagLib::String &s2);
+TAGLIB_EXPORT TagLib::String operator+(const char *s1, const TagLib::String &s2);
 
 /*!
  * \relates TagLib::String
  *
  * Concatenates \a s1 and \a s2 and returns the result as a string.
  */
-TAGLIB_EXPORT const TagLib::String operator+(const TagLib::String &s1, const char *s2);
+TAGLIB_EXPORT TagLib::String operator+(const TagLib::String &s1, const char *s2);
 
 
 /*!
